@@ -15,25 +15,19 @@ GNU_FPOST_ATTR
 
 #if CONFIG_POST & CONFIG_SYS_POST_FPU
 
-int fpu_post_test_math4 (void)
+int fpu_post_test_math3 (void)
 {
-	volatile float reale = 1.0f;
-	volatile float oneplus;
-	int i;
+	volatile long double dfrom = 1.1;
+	volatile long double m1;
+	volatile long double m2;
+	volatile unsigned long mant_long;
 
-	if (sizeof (float) != 4)
-		return 0;
+	m1 = dfrom / 2.0;
+	m2 = m1 * 4294967296.0;
+	mant_long = ((unsigned long) m2) & 0xffffffff;
 
-	for (i = 0; ; i++)
-	{
-		oneplus = 1.0f + reale;
-		if (oneplus == 1.0f)
-			break;
-		reale = reale / 2.0f;
-	}
-	/* Assumes ieee754 accurate arithmetic above.  */
-	if (i != 24) {
-		post_log ("Error in FPU math4 test\n");
+	if (mant_long != 0x8ccccccc) {
+		post_log ("Error in FPU math3 test\n");
 		return -1;
 	}
 	return 0;
